/*
 * Copyright (c) 2007 Chun-Koo Park
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*------------------------------------------------------------------------------
 * Copyright (C) 2003-2006 Ben van Klinken and the CLucene Team
 * 
 * Distributable under the terms of either the Apache License (Version 2.0) or
 * the GNU Lesser General Public License, as specified in the COPYING file.
 ------------------------------------------------------------------------------*/
#include "CLucene.h"
#include "ConvertUtil.h"
#include "CJKAnalyzer.h"

using namespace std;
using namespace lucene::index;
using namespace lucene::analysis;
using namespace lucene::util;
using namespace lucene::store;
using namespace lucene::document;
using namespace lucene::queryParser;
using namespace lucene::document;
using namespace lucene::search;

#define CURRENTMILLISEC() lucene::util::Misc::currentTimeMillis()

Document* TestDocument(const char *f, const char *txt)
{
    printf("adding doc: %s - %s\n", f, txt);
    Document* doc = _CLNEW Document();

    TCHAR tf[CL_MAX_DIR];
    ConvertToUTF32(tf,f,CL_MAX_DIR);
    doc->add( *Field::Keyword(_T("id"), tf ) );

    {
        TCHAR tt[CL_MAX_DIR];
        ConvertToUTF32(tt,txt,CL_MAX_DIR);

        doc->add( *Field::Text(_T("contents"),tt) );
    }

    return doc;
}

void IndexDocs(const char *target)
{
    IndexWriter* writer = NULL;
    cjk::CJKAnalyzer an;

    writer = _CLNEW IndexWriter( target ,&an, true);
    writer->setMaxFieldLength(IndexWriter::DEFAULT_MAX_FIELD_LENGTH);

    uint64_t stms = CURRENTMILLISEC();

    {
        Document* doc = TestDocument("doc1", "hahaha 한글단어 hohoho 비 bye 검색엔진");
        writer->addDocument(doc);
        _CLDELETE(doc);
    }
    {
        Document* doc = TestDocument("doc2", "hello zaza 한글 김현정 김건 건모 검색");
        writer->addDocument(doc);
        _CLDELETE(doc);
    }
    {
        Document* doc = TestDocument("doc3", "goodbye 김건모 서영은 검색 엔진 SG워너비");
        writer->addDocument(doc);
        _CLDELETE(doc);
    }

    writer->optimize();
    writer->close();
    _CLDELETE(writer);

    printf("Indexing took: %ld ms.\n\n", CURRENTMILLISEC() - stms);
}

void SearchDocs(const char* index)
{
    cjk::CJKAnalyzer analyzer;
    char line[80];
    TCHAR tline[80];
    const TCHAR* buf;
#define USE_RAMDIR
#ifdef USE_RAMDIR
    RAMDirectory *pRamDirectory = new RAMDirectory(index);
    IndexSearcher s(pRamDirectory);
#else
    IndexSearcher s(index);
#endif

    while (1) {
        printf("Enter query string: ");
        fgets(line,80,stdin);
        line[strlen(line)-1]=0;

        if ( strlen(line) == 0 )
            break;
        ConvertToUTF32(tline,line,80);

        Query* q = QueryParser::parse(tline,_T("contents"),&analyzer);

        buf = q->toString(_T("contents"));
        printf("Searching for: %s\n\n", ConvertToString(buf).c_str());
        _CLDELETE_CARRAY(buf); // XXX

        uint64_t stms = CURRENTMILLISEC();
        Hits* h = s.search(q);
        uint64_t srch = CURRENTMILLISEC() - stms;

        stms = CURRENTMILLISEC();

        for ( int32_t i=0;i<h->length();i++ ){
            Document* doc = &h->doc(i);
            const TCHAR* ibuf = doc->get(_T("id"));
            const TCHAR* cbuf = doc->get(_T("contents"));
            printf("%d. %s - %s (%f)\n", i, ConvertToString(ibuf).c_str(), ConvertToString(cbuf).c_str(), h->score(i));
        }

        printf("\n\nSearch took: %ld ms.\n", srch);
        printf("Screen dump took: %ld ms.\n\n", CURRENTMILLISEC() - stms);

        _CLDELETE(h);
        _CLDELETE(q);

    }
#ifdef USE_RAMDIR
    _CLDELETE(pRamDirectory);
#endif
    s.close();
}

int main(void)
{
    try {
        char *indexdir = "testindex";

        IndexDocs(indexdir);
        SearchDocs(indexdir);

    } catch (CLuceneError& err) {
        printf(err.what());
    } catch (...) {
        puts("Unknown exception");
    }

    _lucene_shutdown();

    return 0;
}
