/*
 * Copyright (c) 2007 Chun-Koo Park
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "CLucene/StdHeader.h"
#include "KoreanStemFilter.h"

#include "CLucene/analysis/AnalysisHeader.h"
#include "CLucene/analysis/Analyzers.h"
#include "CJKTokenizerConstants.h"
#include "CLucene/util/StringBuffer.h"
#include "ConvertUtil.h"

CL_NS_USE(analysis)
CL_NS_USE(util)
CL_NS_USE2(analysis,cjk)
CL_NS_DEF2(analysis,korean)

  KoreanStemFilter::KoreanStemFilter(TokenStream* in, bool deleteTokenStream):
    TokenFilter(in, deleteTokenStream)
    , curToken(NULL)
    , curTokenOffset(0)
  {
  }

  KoreanStemFilter::~KoreanStemFilter(){
    if (curToken) {
      delete curToken;
      curToken = NULL;
    }
  }

  bool KoreanStemFilter::next(Token* t) {
    if (curToken) {
      int start = curToken->startOffset();
      int end = curToken->endOffset();

      //printf("curTokenOffset+2 (%d) > %d\n", curTokenOffset+2, end-start);

      if (curTokenOffset+2 <= end-start) {

        t->growBuffer(LUCENE_MAX_WORD_LEN+1);

        StringBuffer str(t->_termText,t->bufferLength(),true);
        str.appendChar(curToken->_termText[curTokenOffset]);
        str.appendChar(curToken->_termText[curTokenOffset+1]);
        str.getBuffer();

        t->setStartOffset(start+curTokenOffset);
        t->setEndOffset(start+curTokenOffset+2);
        t->setType(curToken->type());

        curTokenOffset++;
        //printf("KoreanStemFilter: next4:%s (%s)\n", ConvertToString(t->_termText).c_str(), ConvertToString(t->type()).c_str());
        return true;
      } else {
        delete curToken;
        curToken = NULL;
      }
    }

    if (!input->next(t))
      return false;

    const TCHAR* type = t->type();

    if ( _tcscmp(type, _T("<CJK>")) != 0) {
      // CJK 아니면 input->next() 받은거 그대로 리턴
      //printf("KoreanStemFilter: next1:%s (%s)\n", ConvertToString(t->_termText).c_str(), ConvertToString(t->type()).c_str());
      return true;
    }

    int s = t->startOffset();
    int e = t->endOffset();

    if (e - s <= 2) {
      // 두글자 이하는 그대로 토근으로 반환
      //printf("KoreanStemFilter: next2:%s (%s)\n", ConvertToString(t->_termText).c_str(), ConvertToString(t->type()).c_str());
      return true;
    }

    //printf("s,e = (%d,%d)\n", s, e);

    {
      curToken = _CLNEW Token();

      curToken->growBuffer(LUCENE_MAX_WORD_LEN+1);

      StringBuffer str(curToken->_termText,curToken->bufferLength(),true);
      str.append(t->_termText);
      str.getBuffer();

        curToken->setStartOffset(s);
        curToken->setEndOffset(e);
        curToken->setType(t->type());
        curToken->resetTermTextLen();
    }

    curTokenOffset = 0;
    {
        int start = curToken->startOffset();
        //int end = curToken->endOffset();

        //printf("curTokenOffset+2 (%d) > %d\n", curTokenOffset+2, end-start);

        //if (curTokenOffset+2 > end-start) break;

        t->growBuffer(LUCENE_MAX_WORD_LEN+1);

        StringBuffer str(t->_termText,t->bufferLength(),true);
        str.appendChar(curToken->_termText[curTokenOffset]);
        str.appendChar(curToken->_termText[curTokenOffset+1]);
        str.getBuffer();

        t->setStartOffset(start+curTokenOffset);
        t->setEndOffset(start+curTokenOffset+2);
        t->setType(curToken->type());

        curTokenOffset++;
        //printf("KoreanStemFilter: next3:%s (%s)\n", ConvertToString(t->_termText).c_str(), ConvertToString(t->type()).c_str());
    }

    return true;
  }

CL_NS_END2
