#include "CLucene/StdHeader.h"
#include "CLucene.h"
#include "CLucene/util/Reader.h"
#include "CLucene/util/Misc.h"
#include "CLucene/util/dirent.h"
#include "ConvertUtil.h"
#include "CJKAnalyzer.h"
#include <string>

using namespace std;
using namespace lucene::index;
using namespace lucene::analysis;
using namespace lucene::util;
using namespace lucene::store;
using namespace lucene::document;
using namespace lucene::queryParser;
using namespace lucene::search;


#define CURRENTMILLISEC() lucene::util::Misc::currentTimeMillis()


Document* TestDocument(const char *f, const char *txt)
{
    Document* doc = _CLNEW Document();

    TCHAR tf[CL_MAX_DIR];
    ConvertToUTF32(tf,f,CL_MAX_DIR);
    doc->add( *Field::Keyword(_T("id"), tf ) );

    {
        TCHAR tt[CL_MAX_DIR];
        ConvertToUTF32(tt,txt,CL_MAX_DIR);

        doc->add( *Field::Text(_T("contents"),tt) );
    }

    return doc;
}

static void AddDoc(IndexWriter *writer, const string &key, const string &line)
{
	if (line.empty()) return;
	Document* doc = TestDocument(key.c_str(), line.c_str());
	writer->addDocument(doc);
	_CLDELETE(doc);
}

void IndexDocs(const string &indexPath, const string &txt)
{
    IndexWriter* writer = NULL;
    cjk::CJKAnalyzer an;
	string lines = txt;

    writer = _CLNEW IndexWriter(indexPath.c_str(), &an, true);
    writer->setMaxFieldLength(IndexWriter::DEFAULT_MAX_FIELD_LENGTH);

    uint64_t stms = CURRENTMILLISEC();
	int linenum = 1;

	while (! lines.empty()) {
		char buf[10];
		string::size_type pos = lines.find("\r\n");

		_snprintf(buf, 10, "line%d", linenum);

		if (pos == string::npos) {
			AddDoc(writer, buf, lines);
			break;
		} else {
			AddDoc(writer, buf, lines.substr(0, pos));
			lines = lines.substr(pos+2);
		}


		linenum++;
	}

    writer->optimize();
    writer->close();
    _CLDELETE(writer);

    printf("Indexing took: %ld ms.\n\n", CURRENTMILLISEC() - stms);
}

string SearchDocs(const string &indexPath, const string &query)
{
    cjk::CJKAnalyzer analyzer;
	string ret;
	
//#define USE_RAMDIR
#ifdef USE_RAMDIR
    RAMDirectory *pRamDirectory = new RAMDirectory(indexPath.c_str());
    IndexSearcher s(pRamDirectory);
#else
    IndexSearcher s(indexPath.c_str());
#endif

	TCHAR tf[CL_MAX_DIR];
	ConvertToUTF32(tf, query.c_str(), CL_MAX_DIR);

    {
        Query* q = QueryParser::parse(tf, _T("contents"), &analyzer);

		TCHAR* buf;
		buf = q->toString(_T("contents"));

		//OutputDebugString(query.c_str());
        //OutputDebugString(ConvertToString(buf).c_str());
        _CLDELETE_CARRAY(buf);

        uint64_t stms = CURRENTMILLISEC();
        Hits* h = s.search(q);
        uint64_t srch = CURRENTMILLISEC() - stms;

        stms = CURRENTMILLISEC();

        for ( int32_t i=0;i<h->length();i++ ){
			char buf[1024];
            Document* doc = &h->doc(i);
            const TCHAR* ibuf = doc->get(_T("id"));
            const TCHAR* cbuf = doc->get(_T("contents"));
			_snprintf(buf, 1024, "%d. %s - %s", i+1, ConvertToString(ibuf).c_str(), ConvertToString(cbuf).c_str(), h->score(i));
			ret += string(buf) + "\r\n";
        }

        //printf("\n\nSearch took: %ld ms.\n", srch);
        //printf("Screen dump took: %ld ms.\n\n", CURRENTMILLISEC() - stms);

        _CLDELETE(h);
        _CLDELETE(q);

    }
#ifdef USE_RAMDIR
    _CLDELETE(pRamDirectory);
#endif
    s.close();

	return ret;
}
